@@ -8,6 +8,7 @@ use App::Netdisco::Util::DNS ':all';
 use NetAddr::IP::Lite ':lower';
 use Encode;
 use Try::Tiny;
+use Net::MAC;
 
 use base 'Exporter';
 our @EXPORT = ();
@@ -168,7 +169,7 @@ sub store_device {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->device_ips->delete;
-    debug sprintf ' [%s] device - removed %s aliases',
+    debug sprintf ' [%s] device - removed %d aliases',
       $device->ip, $gone;
     $device->update_or_insert(undef, {for => 'update'});
     $device->device_ips->populate($resolved_aliases);
@@ -204,8 +205,8 @@ sub store_interfaces {
   my $i_duplex_admin = $snmp->i_duplex_admin;
   my $i_stp_state    = $snmp->i_stp_state;
   my $i_vlan         = $snmp->i_vlan;
-  my $i_pvid         = $snmp->i_pvid;
   my $i_lastchange   = $snmp->i_lastchange;
+  my $agg_ports      = $snmp->agg_ports;
 
   # clear the cached uptime and get a new one
   my $dev_uptime = $snmp->load_uptime;
@@ -220,7 +221,7 @@ sub store_interfaces {
 
       if (not $port) {
           debug sprintf ' [%s] interfaces - ignoring %s (no port mapping)',
-            $device->ip, $port;
+            $device->ip, $entry;
           next;
       }
 
@@ -277,14 +278,26 @@ sub store_interfaces {
           stp          => $i_stp_state->{$entry},
           type         => $i_type->{$entry},
           vlan         => $i_vlan->{$entry},
-          pvid         => $i_pvid->{$entry},
+          pvid         => $i_vlan->{$entry},
+          is_master    => 'false',
+          slave_of     => undef,
           lastchange   => $lc,
       };
   }
 
+  # must do this after building %interfaces so that we can set is_master
+  foreach my $sidx (keys %$agg_ports) {
+      my $slave  = $interfaces->{$sidx} or next;
+      my $master = $interfaces->{ $agg_ports->{$sidx} } or next;
+      next unless exists $interfaces{$slave} and exists $interfaces{$master};
+
+      $interfaces{$slave}->{slave_of} = $master;
+      $interfaces{$master}->{is_master} = 'true';
+  }
+
   schema('netdisco')->resultset('DevicePort')->txn_do_locked(sub {
     my $gone = $device->ports->delete({keep_nodes => 1});
-    debug sprintf ' [%s] interfaces - removed %s interfaces',
+    debug sprintf ' [%s] interfaces - removed %d interfaces',
       $device->ip, $gone;
     $device->update_or_insert(undef, {for => 'update'});
     $device->ports->populate([values %interfaces]);
@@ -323,7 +336,7 @@ sub store_wireless {
 
       if (not $port) {
           debug sprintf ' [%s] wireless - ignoring %s (no port mapping)',
-            $device->ip, $port;
+            $device->ip, $iid;
           next;
       }
 
@@ -337,7 +350,7 @@ sub store_wireless {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->ssids->delete;
-    debug sprintf ' [%s] wireless - removed %s SSIDs',
+    debug sprintf ' [%s] wireless - removed %d SSIDs',
       $device->ip, $gone;
     $device->ssids->populate(\@ssids);
     debug sprintf ' [%s] wireless - added %d new SSIDs',
@@ -351,7 +364,7 @@ sub store_wireless {
 
       if (not $port) {
           debug sprintf ' [%s] wireless - ignoring %s (no port mapping)',
-            $device->ip, $port;
+            $device->ip, $entry;
           next;
       }
 
@@ -364,7 +377,7 @@ sub store_wireless {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->wireless_ports->delete;
-    debug sprintf ' [%s] wireless - removed %s wireless channels',
+    debug sprintf ' [%s] wireless - removed %d wireless channels',
       $device->ip, $gone;
     $device->wireless_ports->populate(\@channels);
     debug sprintf ' [%s] wireless - added %d new wireless channels',
@@ -439,7 +452,7 @@ sub store_vlans {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->vlans->delete;
-    debug sprintf ' [%s] vlans - removed %s device VLANs',
+    debug sprintf ' [%s] vlans - removed %d device VLANs',
       $device->ip, $gone;
     $device->vlans->populate(\@devicevlans);
     debug sprintf ' [%s] vlans - added %d new device VLANs',
@@ -448,7 +461,7 @@ sub store_vlans {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->port_vlans->delete;
-    debug sprintf ' [%s] vlans - removed %s port VLANs',
+    debug sprintf ' [%s] vlans - removed %d port VLANs',
       $device->ip, $gone;
     $device->port_vlans->populate(\@portvlans);
     debug sprintf ' [%s] vlans - added %d new port VLANs',
@@ -515,7 +528,7 @@ sub store_power {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->power_modules->delete;
-    debug sprintf ' [%s] power - removed %s power modules',
+    debug sprintf ' [%s] power - removed %d power modules',
       $device->ip, $gone;
     $device->power_modules->populate(\@devicepower);
     debug sprintf ' [%s] power - added %d new power modules',
@@ -524,7 +537,7 @@ sub store_power {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->powered_ports->delete;
-    debug sprintf ' [%s] power - removed %s PoE capable ports',
+    debug sprintf ' [%s] power - removed %d PoE capable ports',
       $device->ip, $gone;
     $device->powered_ports->populate(\@portpower);
     debug sprintf ' [%s] power - added %d new PoE capable ports',
@@ -588,7 +601,7 @@ sub store_modules {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->modules->delete;
-    debug sprintf ' [%s] modules - removed %s chassis modules',
+    debug sprintf ' [%s] modules - removed %d chassis modules',
       $device->ip, $gone;
     $device->modules->populate(\@modules);
     debug sprintf ' [%s] modules - added %d new chassis modules',
@@ -617,7 +630,7 @@ sub store_neighbors {
   my ($device, $snmp) = @_;
   my @to_discover = ();
 
-  # first allow any manually configred topology to be set
+  # first allow any manually configured topology to be set
   _set_manual_topology($device, $snmp);
 
   my $c_ip = $snmp->c_ip;
@@ -631,6 +644,7 @@ sub store_neighbors {
   my $c_port     = $snmp->c_port;
   my $c_id       = $snmp->c_id;
   my $c_platform = $snmp->c_platform;
+  my $c_cap      = $snmp->c_cap;
 
   foreach my $entry (keys %$c_ip) {
       my $port = $interfaces->{ $c_if->{$entry} };
@@ -645,6 +659,7 @@ sub store_neighbors {
       my $remote_port = undef;
       my $remote_type = $c_platform->{$entry};
       my $remote_id   = Encode::decode('UTF-8', $c_id->{$entry});
+      my $remote_cap  = $c_cap->{$entry} || [];
 
       next unless $remote_ip;
 
@@ -661,6 +676,13 @@ sub store_neighbors {
                 ' [%s] neigh - bad address %s on port %s, searching for %s instead',
                 $device->ip, $remote_ip, $port, $remote_id;
 
+              if (!defined $neigh) {
+                  my $mac = Net::MAC->new(mac => $remote_id, 'die' => 0, verbose => 0);
+                  if (not $mac->get_error) {
+                      $neigh = $devices->single({mac => $remote_id});
+                  }
+              }
+
               if (!defined $neigh) {
                   (my $shortid = $remote_id) =~ s/\..*//;
                   $neigh = $devices->single({name => { -ilike => "${shortid}%" }});
@@ -684,13 +706,21 @@ sub store_neighbors {
           }
       }
 
-      # IP Phone detection type fixup
-      if (defined $remote_type and $remote_type =~ m/(mitel.5\d{3})/i) {
-          $remote_type = 'IP Phone - '. $remote_type
-            if $remote_type !~ /ip phone/i;
-      }
-      else {
-          $remote_type ||= '';
+      # IP Phone and WAP detection type fixup
+      if (defined $remote_type) {
+          my $phone_flag = grep {/phone/i} @$remote_cap;
+          my $ap_flag    = grep {/wlanAccessPoint/} @$remote_cap;
+
+          if ($phone_flag or $remote_type =~ m/(mitel.5\d{3})/i) {
+              $remote_type = 'IP Phone: '. $remote_type
+              if $remote_type !~ /ip phone/i;
+          }
+          elsif ($ap_flag) {
+              $remote_type = 'AP: '. $remote_type;
+          }
+          else {
+              $remote_type ||= '';
+          }
       }
 
       # hack for devices seeing multiple neighbors on the port
@@ -703,7 +733,7 @@ sub store_neighbors {
               foreach my $n (@$remote_ip) {
                   debug sprintf
                     ' [%s] neigh - adding neighbor %s, type [%s], on %s to discovery queue',
-                    $device->ip, $n, $remote_type, $port;
+                    $device->ip, $n, ($remote_type || ''), $port;
                   push @to_discover, [$n, $remote_type];
               }
           }
@@ -717,7 +747,7 @@ sub store_neighbors {
           if (wantarray) {
               debug sprintf
                 ' [%s] neigh - adding neighbor %s, type [%s], on %s to discovery queue',
-                $device->ip, $remote_ip, $remote_type, $port;
+                $device->ip, $remote_ip, ($remote_type || ''), $port;
               push @to_discover, [$remote_ip, $remote_type];
           }
 
@@ -757,6 +787,26 @@ sub store_neighbors {
           is_uplink   => \"true",
           manual_topo => \"false",
       });
+
+      if (defined $portrow->slave_of and
+          my $master = schema('netdisco')->resultset('DevicePort')
+              ->single({ip => $device->ip, port => $portrow->slave_of})) {
+
+          if (not ($portrow->is_master or defined $master->slave_of)) {
+              # TODO needs refactoring - this is quite expensive
+              my $peer = schema('netdisco')->resultset('DevicePort')->find({
+                  ip   => $portrow->neighbor->ip,
+                  port => $portrow->remote_port,
+              }) if $portrow->neighbor;
+
+              $master->update({
+                  remote_ip => ($peer ? $peer->ip : $remote_ip),
+                  remote_port => ($peer ? $peer->slave_of : undef ),
+                  is_uplink => \"true",
+                  manual_topo => \"false",
+              });
+          }
+      }
   }
 
   return @to_discover;
@@ -844,7 +894,7 @@ sub discover_new_neighbors {
       if (not is_discoverable($device, $remote_type)) {
           debug sprintf
             ' queue - %s, type [%s] excluded by discover_* config',
-            $ip, $remote_type;
+            $ip, ($remote_type || '');
           next;
       }
 
