@@ -10,6 +10,7 @@ use Socket6 (); # to ensure dependency is met
 use HTML::Entities (); # to ensure dependency is met
 use URI::QueryParam (); # part of URI, to add helper methods
 use Path::Class 'dir';
+use App::Netdisco::Util::Web 'interval_to_daterange';
 
 use App::Netdisco::Web::AuthN;
 use App::Netdisco::Web::Static;
@@ -19,7 +20,8 @@ use App::Netdisco::Web::Report;
 use App::Netdisco::Web::AdminTask;
 use App::Netdisco::Web::TypeAhead;
 use App::Netdisco::Web::PortControl;
-use App::Netdisco::Web::About;
+use App::Netdisco::Web::Statistics;
+use App::Netdisco::Web::Password;
 
 sub _load_web_plugins {
   my $plugin_list = shift;
@@ -62,6 +64,9 @@ hook 'before_template' => sub {
     # access to logged in user's roles
     $tokens->{user_has_role}  = sub { user_has_role(@_) };
 
+    # create date ranges from within templates
+    $tokens->{to_daterange}  = sub { interval_to_daterange(@_) };
+
     # fix Plugin Template Variables to be only path+query
     $tokens->{$_} = $tokens->{$_}->path_query
       for qw/search_node search_device device_ports/;
