@@ -21,8 +21,8 @@ hook 'before' => sub {
     { name => 'c_speed',       label => 'Speed',             default => ''   },
     { name => 'c_mac',         label => 'Port MAC',          default => ''   },
     { name => 'c_mtu',         label => 'MTU',               default => ''   },
-    { name => 'c_vlan',        label => 'Native VLAN',       default => 'on' },
-    { name => 'c_vmember',     label => 'Tagged VLANs',      default => 'on' },
+    { name => 'c_pvid',        label => 'Native VLAN',       default => 'on' },
+    { name => 'c_vmember',     label => 'VLAN Membership',   default => 'on' },
     { name => 'c_power',       label => 'PoE',               default => ''   },
     { name => 'c_nodes',       label => 'Connected Nodes',   default => ''   },
     { name => 'c_neighbors',   label => 'Connected Devices', default => 'on' },
@@ -48,6 +48,9 @@ hook 'before' => sub {
   var('connected_properties' => [
     { name => 'n_age',      label => 'Age Stamp',     default => ''   },
     { name => 'n_ip',       label => 'IP Address',    default => 'on' },
+    { name => 'n_netbios',  label => 'NetBIOS',       default => 'on' },
+    { name => 'n_ssid',     label => 'SSID',          default => 'on' },
+    { name => 'n_vendor',   label => 'Vendor',        default => ''   },
     { name => 'n_archived', label => 'Archived Data', default => ''   },
   ]);
 
@@ -146,25 +149,34 @@ hook 'before_template' => sub {
   my $self_uri = uri_for(request->path, scalar params);
   $self_uri->query_param_delete('q');
   $self_uri->query_param_delete('f');
+  $self_uri->query_param_delete('prefer');
   $tokens->{self_options} = $self_uri->query_form_hash;
 };
 
 get '/device' => require_login sub {
     my $q = param('q');
-    my $dev = schema('netdisco')->resultset('Device')->single({
+    my $schema = schema('netdisco')->resultset('Device');
+
+    # we are passed either dns or ip
+    my $dev = $schema->search({
         -or => [
             \[ 'host(me.ip) = ?' => [ bind_value => $q ] ],
             'me.dns' => $q,
         ],
     });
 
-    if (!defined $dev) {
+    if ($dev->count == 0) {
         return redirect uri_for('/', {nosuchdevice => 1})->path_query;
     }
 
+    # if passed dns, need to check for duplicates
+    # and use only ip for q param, if there are duplicates.
+    my $first = $dev->first;
+    my $others = ($schema->search({dns => $first->dns})->count() - 1);
+
     params->{'tab'} ||= 'details';
     template 'device', {
-      d => $dev,
+      display_name => ($others ? $first->ip : $first->dns),
       device => params->{'tab'},
     };
 };
